#!BPY
"""
Name: 'Uni'
Blender: 241
Group: 'Mesh'
Tip: 'It makes a vertex at the center of the selected surface.'
"""

__author__ = 'Nabubun'
__version__ = '1.0 2006/06/23'
__url__ = ["", "", "", ""]
__email__ = ["Nabubun", ""]
__bpydoc__ = """\
It makes a vertex at the center of the selected surface.

Shortcuts:<br>
	None.

Supported:<br>
	Window.

Known issues:<br>

Notes:<br>

"""

import Blender
from Blender import *

def canselSelect():
	global me
	for i in me.faces:
		i.sel = 0
	for i in me.verts:
		i.sel = 0

def selectVerts(ver):
	for i in ver:
		i.sel = 1

def selectFaces(fa):
	for i in fa:
		i.sel = 1
		for j in i.v:
			j.sel = 1

def getPoint(fa):
	x = y = z = 0.
	le = len(fa.v)
	for i in fa.v:
		x += i.co.x
		y += i.co.y
		z += i.co.z
	x /= le
	y /= le
	z /= le
	return([x, y, z])

def getPoint2(fa):
	ce = getPoint(fa)
	p = []
	for i in fa.v:
		x = ce[0] - (ce[0] - i.co.x) * b_ra.val
		y = ce[1] - (ce[1] - i.co.y) * b_ra.val
		z = ce[2] - (ce[2] - i.co.z) * b_ra.val
		p.append([x, y, z])
	return(p)

def makeVer(fa):
	p = getPoint(fa)
	ver = NMesh.Vert(p[0], p[1], p[2])
	return(ver)

def makeVer2(fa):
	p = getPoint2(fa)
	vs = []
	for i in p:
		v = NMesh.Vert(i[0], i[1], i[2])
		vs.append(v)
	return(vs)

def makeFa(fa):
	global me
	ln = len(fa.v)
	ver = makeVer(fa)
	me.verts.append(ver)
	v = [ver, fa.v[0], fa.v[1]]
	f = NMesh.Face(v)
	me.addFace(f)
	v = [ver, fa.v[1], fa.v[2]]
	f = NMesh.Face(v)
	me.addFace(f)
	if ln == 3:
		v = [ver, fa.v[2], fa.v[0]]
		f = NMesh.Face(v)
		me.addFace(f)
	else:
		v = [ver, fa.v[2], fa.v[3]]
		f = NMesh.Face(v)
		me.addFace(f)
		v = [ver, fa.v[3], fa.v[0]]
		f = NMesh.Face(v)
		me.addFace(f)
	me.removeFace(fa)
	return(ver)

def makeFa2(fa):
	global me
	ln = len(fa.v)
	vs = makeVer2(fa)
	for i in vs:
		me.verts.append(i)
	# - make face -
	face = NMesh.Face(vs)
	me.addFace(face)
	# - meke fase sub -
	v = [vs[0], vs[1], fa.v[1], fa.v[0]]
	f = NMesh.Face(v)
	me.addFace(f)
	v = [vs[1], vs[2], fa.v[2], fa.v[1]]
	f = NMesh.Face(v)
	me.addFace(f)
	if ln == 3:
		v = [vs[2], vs[0], fa.v[0], fa.v[2]]
		f = NMesh.Face(v)
		me.addFace(f)
	else:
		v = [vs[2], vs[3], fa.v[3], fa.v[2]]
		f = NMesh.Face(v)
		me.addFace(f)
		v = [vs[3], vs[0], fa.v[0], fa.v[3]]
		f = NMesh.Face(v)
		me.addFace(f)
	me.removeFace(fa)
	return(face)


def getSelectedFaaces():
	global me
	s_fa = []
	for i in me.faces:
		if i.sel:
			s_fa.append(i)
	if s_fa == []:
		Draw.PupMenu("Choose equal to or more than one faces.")
	return(s_fa)

def uni():
	global me
	#print "Uni start"
	#print me.faces
	s_fa = getSelectedFaaces()
	if s_fa  == []:
		return
	vers = []
	for i in s_fa:
		v = makeFa(i)
		vers.append(v)
	canselSelect()
	selectVerts(vers)
	me.update()
	return(True)

def uni2():
	global me
	#print "Uni2 start"
	s_fa = getSelectedFaaces()
	if s_fa  == []:
		return
	fs = []
	for i in s_fa:
		f = makeFa2(i)
		fs.append(f)
	canselSelect()
	selectFaces(fs)
	me.update()
	return(True)

def getNMeshName():
	scene = Scene.GetCurrent()
	if not scene:
		Draw.PupMenu("Error: There is not a scene.")
		return
	ob = scene.getActiveObject()
	#print ob
	if ob.getType() != 'Mesh':
		Draw.PupMenu("Error: Activate meshes.")
		return
	me_name = ob.getData(True)
	#print me_name
	return(me_name)


#----- start -----
me_name = getNMeshName()
if me_name:
	mode = Window.EditMode()
	if mode:
		b_fa = Draw.Create(0)
		b_ra = Draw.Create(0.5)
		b_bl = []
		b_bl.append(("Meke face", b_fa))
		b_bl.append(("Ratio:", b_ra, 0.0, 1.0))
		b_res = Draw.PupBlock("Uni...", b_bl)
		if b_res:
			Window.EditMode(0)
			print "Start Uni."
			print "Object neme is", me_name, "."
			me = NMesh.GetRaw(me_name)
			if (b_fa.val == 0) | (b_ra.val == 0.0):
				res = uni()
			else:
				res = uni2()
			print "End Uni."
			if res:
				#NMesh.PutRaw(me, me_name)
				Window.EditMode(1)
				Window.EditMode(0)
			Window.EditMode(1)
			Redraw()
	else:
		Draw.PupMenu("Have an edit mode.")
else:
	print "Error"